/*
  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.

  Copyright (c) 2000-2008 SpringCard - www.springcard.com
  See COPYING for copyright and legal info

  pssc_low.c
  ----------
  Low level functions

 */

#include "springcard_i.h"

LONG SCARD_PowerUp_Raw(SCARD_HANDLE_ST *card)
{
  LONG rc;
  
  if ((gcr_family == GCR_LITE) || (gcr_family == GCR_LITE_PRO) || (card->ask_protocol == SCARD_PROTOCOL_RAW))
  {
    /* GCR LITE and LITE PRO can't handle memory cards, we need an "emulated driver" */
    rc = GCR_SetMode(card, ISO_MODE);
    if (rc == SCARD_S_SUCCESS)
      rc = GCR_SetType(card, RAW_TYPE);
    if (rc == SCARD_S_SUCCESS)
    	rc = GCR_PowerUp(card);

  } else    
  {
    /* Regular GCR can handle memory cards in hardware, card type is */
    /* selected by the calling application                           */
    BYTE    raw_type = (BYTE) (card->ask_protocol % 0x0100);
    
    rc = GCR_SetMode(card, ISO_MODE);
    if (rc == SCARD_S_SUCCESS)
      rc = GCR_SetType(card, raw_type);
    if (rc == SCARD_S_SUCCESS)
    	rc = GCR_PowerUp(card);
  }

  if (rc == (SCARD_W_UNRESPONSIVE_CARD))
    rc = SCARD_S_SUCCESS;   /* Not so bad, a memory card can be mute ! */
  
  return rc;
}

LONG SCARD_PowerUp(SCARD_HANDLE_ST *card)
{
  LONG rc;
     
  if ((gcr_family == GCR_LITE) || (gcr_family == GCR_POS))
  {
    /* GemCore Lite : select EMV or ISO mode before power up */
    if (card->ask_protocol & SCARD_PROTOCOL_EMV)
      rc = GCR_SetMode(card, 0x45);
    else
      rc = GCR_SetMode(card, 0x47);
    if (rc != SCARD_S_SUCCESS)
      return rc;
  }

  if (gcr_family != GCR_LITE_PRO)
  {
    /* On GemCore Lite Pro, type must be changed later */
    if (card->ask_protocol & SCARD_PROTOCOL_HIS)
      rc = GCR_SetType(card, 0x12);
    else
      rc = GCR_SetType(card, 0x02);
    if (rc != SCARD_S_SUCCESS)
      return rc;
  }
    
  /* First power-up */
  if (card->ask_protocol & SCARD_PROTOCOL_EMV)
  {
    /* EMV : no provide parameter for power-up */
    rc = GCR_PowerUp(card);
  } else
  {
    /* Not EMV */
    if (card->ask_protocol & SCARD_PROTOCOL_PTS)
      rc = GCR_PowerUpEx(card, 0x23);
    else
      rc = GCR_PowerUpEx(card, 0x13);
  }
    
  if (rc != SCARD_S_SUCCESS)
  {
    /* ISO Mode can be selected only after failure in EMV mode */
    rc = GCR_SetMode(card, 0x47);
    if (rc != SCARD_S_SUCCESS)
      return rc;

    /* Now set type  */
    if (card->ask_protocol & SCARD_PROTOCOL_HIS)
      rc = GCR_SetType(card, 0x12);
    else
      rc = GCR_SetType(card, 0x02);
    if (rc != SCARD_S_SUCCESS)
      return rc;
      
    /* Second power-on */
    if (card->ask_protocol & SCARD_PROTOCOL_PTS)
      rc = GCR_PowerUpEx(card, 0x23);
    else
      rc = GCR_PowerUpEx(card, 0x13);       
  }

  /* Try again with default parameters on error */  
  if ((rc != SCARD_S_SUCCESS) && (card->ask_protocol & SCARD_PROTOCOL_PTS))
  {
    rc = GCR_SetType(card, 0x02);
    if (rc != SCARD_S_SUCCESS)
      return rc;

    rc = GCR_PowerUpEx(card, 0x13);
  }

  return rc;
}

LONG SCARD_Transmit(SCARD_HANDLE_ST *card, BYTE *sendBuffer, DWORD sendLen, BYTE *recvBuffer, DWORD recvSize, DWORD *recvLen)
{
  LONG rc;
  
  if (gcr_family == GCR_LITE)
  {
    /* Card must be T=0 or T=1 */
    if ((card->run_protocol != SCARD_PROTOCOL_T0) && (card->run_protocol != SCARD_PROTOCOL_T1))
    {
			//MessageBox(0, _T("SCardTransmit"), _T("SCARD_E_UNSUPPORTED_FEATURE"), 0);    
      return SCARD_E_UNSUPPORTED_FEATURE;
    }
  }

  if (SCARD_TraceGetLevel() & TR_PCSC_LOW)
  {
    DWORD i;
    
    SCARD_Trace(TR_PCSC_LOW, "Transmit> ");
    for (i=0; i<sendLen; i++)
      SCARD_Trace(TR_PCSC_LOW, "_%02X", sendBuffer[i]);
  }

  rc = GCR_Transmit(card, sendBuffer, sendLen, recvBuffer, recvSize, recvLen);
  if (rc != SCARD_S_SUCCESS)
  {
    SCARD_Trace(TR_PCSC_LOW, "Transmit failed (%08lX)", rc);
    SCARD_Status(card); 
  }
  
  if ((rc == SCARD_S_SUCCESS) && (SCARD_TraceGetLevel() & TR_PCSC_LOW))
  {
    DWORD i;
    
    SCARD_Trace(TR_PCSC_LOW, "Transmit< ");
    for (i=0; i<*recvLen; i++)
      SCARD_Trace(TR_PCSC_LOW, "_%02X", recvBuffer[i]);
    SCARD_Trace(TR_PCSC_LOW, NULL);    
  }
  
  return rc;
}

LONG SCARD_Status(SCARD_HANDLE_ST *card)
{
  LONG rc;
  
  rc = GCR_Status(card);
  if (rc == SCARD_S_SUCCESS)
  {
    if (card->ask_protocol & SCARD_PROTOCOL_RAW)
    {
  	  if ((gcr_family == GCR_LITE) || (gcr_family == GCR_LITE_PRO) || (card->run_protocol == SCARD_PROTOCOL_RAW))
  	  {  	
        if (card->status >= SCARD_SWALLOWED)
        {
          /* The GCR LITE can't power-up a memory card, this is not an error */
          /* We force the RAW protocol for swallowed card                    */
          card->run_protocol = SCARD_PROTOCOL_RAW;
          card->status = SCARD_POWERED;
        }
      }
    }
  }
  
  return rc;  
}

LONG SCARD_PowerDown(SCARD_HANDLE_ST *card)
{ 
  return GCR_PowerDown(card);
}

LONG SCARD_Control(SCARD_HANDLE_ST *card, BYTE *sendBuffer, DWORD sendLen, BYTE *recvBuffer, DWORD recvSize, DWORD *recvLen)
{
  LONG rc;
  
  if (SCARD_TraceGetLevel() & TR_PCSC_LOW)
  {
    DWORD i;
    
    SCARD_Trace(TR_PCSC_LOW, "Control> ");
    for (i=0; i<sendLen; i++)
      SCARD_Trace(TR_PCSC_LOW, "_%02X", sendBuffer[i]);
  }

  rc = GCR_Command(card, sendBuffer, sendLen, recvBuffer, recvSize, recvLen, 0);
  
  if ((rc == SCARD_S_SUCCESS) && (SCARD_TraceGetLevel() & TR_PCSC_LOW))
  {
    DWORD i;
    
    SCARD_Trace(TR_PCSC_LOW, "Control< ");
    for (i=0; i<*recvLen; i++)
      SCARD_Trace(TR_PCSC_LOW, "_%02X", recvBuffer[i]);
    SCARD_Trace(TR_PCSC_LOW, NULL);    
  }
  
  return rc;
}

/*
 * Get the GemCore firmware
 * ------------------------
 * And translate it into SpringCard friendly name
 */
LONG RDR_GetFirmware(SCARD_CONTEXT_ST *context)
{
  BYTE    buffer[32];
  DWORD   recvLen;
  int     version;
  char    *p, *b;
  int     rc;

  recvLen = sizeof(buffer);
  rc = GCR_GetFirmware(&context->desc, buffer, &recvLen);

  if (rc != SCARD_S_SUCCESS)
    return rc;

  buffer[recvLen] = '\0';
  
  SCARD_Trace(TR_GCR_HIGH, "Firmware: %s", buffer);

  if (!strncmp((char *) buffer, "GemCore", 7))
  {
    p = context->names;
    b = (char *) &buffer[7];

    /* First slot: */
    strcpy(p, "SpringCard GCR");
    strcat(p, b);
    strcat(p, " slot A");
    p += strlen(p) + 1;

    /* Second slot? */
    if (context->slot_count > 1)
    {
      strcpy(p, "SpringCard GCR");
      strcat(p, b);
      strcat(p, " slot B");
      p += strlen(p) + 1;
    }

    *p = '\0';
  } else
    return SCARD_E_READER_UNSUPPORTED;
    
  /* Recognize the GemCore family here */ 
  p = &buffer[8];
  while ((*p < '0') || (*p > '9')) p++;
  version = 10 * (*p - '0');
  p++;
  if (*p == '.')
  {
  	p++;
  	if ((*p >= '0') && (*p <= '9'))
  	{
  		version += (*p - '0');
  	}
  }
  
  switch (version)
  {
  	case 11 :
  	case 12 : gcr_family = GCR_ORIGINAL;
  	          SCARD_Trace(TR_GCR_HIGH, "GCR 1.1x or 1.2x");
  	          break;
  	case 13 : gcr_family = GCR_LITE;
  	          SCARD_Trace(TR_GCR_HIGH, "GCR 1.3x LITE");
  	          break;
  	case 14 : gcr_family = GCR_LITE_PRO;
  	          SCARD_Trace(TR_GCR_HIGH, "GCR 1.4x LITE PRO");
  	          break;  	          
  	case 15 : gcr_family = GCR_POS;
  	          SCARD_Trace(TR_GCR_HIGH, "GCR 1.5x POS");
  	          break;
  	default : MessageBox(0, _T("Unsupported GemCore Model"), _T("SpringCard"), 0);
  	          gcr_family = GCR_ORIGINAL; /* Is it the best choice ? */
  	          break;
  }

  return SCARD_S_SUCCESS;
}

LONG RDR_HighSpeed(SCARD_CONTEXT_ST *context)
{
  return GCR_SetBaudrate(&context->desc, 38400);
}

/*
 * Put the reader offline
 * ----------------------
 */
void RDR_Close(SCARD_CONTEXT_ST *context)
{
  SCARD_Trace(TR_GCR_HIGH, "Reader -> OFF");
  SerialClose(&context->desc);
  context->enabled = FALSE;
  context->start_count = 0;
}

/*
 * Wake-up reader
 */
LONG RDR_Resume(SCARD_CONTEXT_ST *context)
{
  return 0;

  if (context->start_count > 0)
  {
    context->start_count++;
    return SCARD_S_SUCCESS;
  }
#ifdef USE_READER_IDLE
  if (SerialOpen(&context->desc, sc_com_name) >= 0)
  {
    if (InitReader(&context->desc) >= 0)
    {
      SCARD_Trace(TR_GCR_HIGH, "Reader wakeup OK");
      context->start_count = 1;
      return SCARD_S_SUCCESS;
    } else
    {
      SCARD_Trace(TR_GCR_HIGH, "Reader wakeup Failed (1)");
      SerialClose(&context->desc);
    }
  } else
  {
    SCARD_Trace(TR_GCR_HIGH, "Reader wakeup Failed (2)");
  }
  return SCARD_E_READER_UNAVAILABLE;
#else
  context->start_count = 1;
  return SCARD_S_SUCCESS;
#endif
}

/*
 * Put reader in idle mode
 * -----------------------
 */
LONG RDR_Suspend(SCARD_CONTEXT_ST *context)
{
  if (context->start_count <= 0)
  {
    return SCARD_S_SUCCESS;
  }
  context->start_count--;
  if (context->start_count == 0)
  {
    SCARD_Trace(TR_GCR_HIGH, "Reader -> sleep");
#ifdef USE_READER_IDLE
    SerialClose(&context->desc);
#endif
    context->start_count = 0;
  }

  return SCARD_S_SUCCESS;
}
