/* SpringCardS10API/S10_GCR_1.4X
 *
 * NAME
 *   SpringCardS10API :: S10 Card specific functions for GemCore Serial Lite PRO V1.4X
 *
 * COPYRIGHT
 *   (c) 2009 SpringCard - www.springcard.com
 *
 * DESCRIPTION
 *   Implementation of S10 functions to read or write the card
 *   For GemCore Serial Lite PRO V1.4X
 *
 **/

#include "springcard_s10_i.h"

/* SpringCardS10API/LITE_PRO_S10_POWER_UP_CMD
 *
 * NAME
 *   LITE_PRO_S10_POWER_UP_CMD
 *
 * DESCRIPTION
 *   Power up the card
 *   For GemCore Serial Lite PRO V1.4X
 *
 **/
static BYTE LITE_PRO_S10_POWER_UP_CMD[] =
{
  0x16, // Synchronous interpreter command

  0x00, // A  = CLA    don't care
  0x12, // R5 = INS    don't care
  0x00, // R6 = P1     don't care
  0x00, // R7 = P2     don't care
  0x00, // R2 = Lin    don't care
  0x00, // R3 = Lout   don't care
  0x02, // 2 lines to follow

  0x11, //       %VCC_ON
  0x12  //       %RESET
};

/* SpringCardS10API/LITE_PRO_S10_READ_DATA_AREA_CMD
 *
 * NAME
 *   LITE_PRO_S10_READ_DATA_AREA_CMD
 *
 * DESCRIPTION
 *   S10 read data raw command
 *   For GemCore Serial Lite PRO V1.4X
 *
 **/
static BYTE LITE_PRO_S10_READ_DATA_AREA_CMD[] =
{
  0x16, // Synchronous interpreter command

  0x00, // A  = CLA    don't care
  0xB0, // R5 = INS    don't care
  0x00, // R6 = P1   = address
  0x00, // R7 = P2     don't care
  0x00, // R2 = Lin    don't care
  0x01, // R3 = Lout = size to read
  23,   // lines to follow

  0x21, //       %CLR_RST             ' reset = 0

// Start command mode
  0x51, //       %SET_IO              ' I/O = 1
  0x71, //       %SET_CLK             ' CLK = 1 
  0x41, //       %CLR_IO              ' I/O = 0
// Command
  0x74, //       MOV      A,0x30      ' A = 0x30 (read main memory command)
  0x30,
  0x93, //       %WRH_R               ' write A
// Address
  0xEE, //       MOV      A,R6        ' A = R6 = P1 (read address)
  0x93, //       %WRH_R               ' write A
// Data
  0x74, //       MOV      A,0x00      ' A = 0x00 (not used)
  0x00,
  0x93, //       %WRH_R               ' write A

// Stop command mode
  0x41, //       %CLR_IO              ' I/O = 0
  0x71, //       %SET_CLK             ' CLK = 1 
  0x51, //       %SET_IO              ' I/O = 1

//Outgoing Data
  0x61, //       %CLR_CLK             ' CLK = 0
		// loop:
  0x53, //       %RDH_R               ' A   = read
  0xF6, //       MOV      @R0,A       ' *R0 = A
  0x08, //       INC      R0          ' R0++
  0xDB, //       DJNZ     R3,0xFB     ' R3--, if R3<>0 goto loop
  0xFB, //                            ' FB = FF - 4 thus move back of 4 instructions

  0xA2, //       %RST_PUL             ' reset card
  0x42  //       %RET_OK              ' data_out = [R4..R0], sw1=0x90, sw2=0x00
};

/* SpringCardS10API/LiteProS10ReadDataArea
 *
 * NAME
 *   LiteProS10ReadDataArea
 *
 * DESCRIPTION
 *   S10 read main memory function using raw exchange
 *   For GemCore Serial Lite PRO V1.4X
 *
 * INPUTS
 *   SCARDHANDLE hCard : handle of the card
 *   BYTE *recv_data   : data received
 *   BYTE addr         : first address to read
 *   BYTE recv_len     : length of data to receive
 *   BYTE status[2]    : 2-byte buffer to receive status code
 *
 * RETURNS
 *   SCARD_S_SUCCESS   : success, card read
 *   Other code if internal or communication error has occured.  
 *
 * NOTES
 *   Maximun length of data to read is 128 bytes
 *
 * SEE ALSO
 *   PosS10ReadDataArea
 *   ScardS10ReadDataArea
 *
 **/
LONG LiteProS10ReadDataArea(SCARDHANDLE hCard, BYTE *recv_data, BYTE addr, BYTE recv_len, BYTE status[2])
{
  LONG SCardLastError;
  BYTE buffer[MAX_BYTE_TO_READ + 2];

  if (recv_len > MAX_BYTE_TO_READ) 
    return SCARD_E_INVALID_PARAMETER;

  if (recv_data == NULL) 
    return SCARD_E_INVALID_PARAMETER;

  /* Power-up (again ?) the card */
  SCardLastError = SCardS10CardDirect(hCard, sizeof(LITE_PRO_S10_POWER_UP_CMD), LITE_PRO_S10_POWER_UP_CMD, sizeof(buffer), buffer, status);
  if (SCardLastError != SCARD_S_SUCCESS)
    return SCardLastError; 

  LITE_PRO_S10_READ_DATA_AREA_CMD[3] = addr;
  LITE_PRO_S10_READ_DATA_AREA_CMD[6] = recv_len;

  SCardLastError = SCardS10CardDirect(hCard, sizeof(LITE_PRO_S10_READ_DATA_AREA_CMD), LITE_PRO_S10_READ_DATA_AREA_CMD, sizeof(buffer), buffer, status);
  if (SCardLastError != SCARD_S_SUCCESS)
    return SCardLastError; 

  memcpy(recv_data, buffer, recv_len);
 
  return SCARD_S_SUCCESS; 
}

/* SpringCardS10API/LITE_PRO_S10_READ_PROTECTION_AREA_CMD
 *
 * NAME
 *   LITE_PRO_S10_READ_PROTECTION_AREA_CMD
 *
 * DESCRIPTION
 *   S10 read protection raw command
 *   For GemCore Serial Lite PRO V1.4X
 *
 **/
static BYTE LITE_PRO_S10_READ_PROTECTION_AREA_CMD[] =
{
  0x16, // Synchronous interpreter command

  0x00, // A  = CLA    don't care
  0xB0, // R5 = INS    don't care
  0x00, // R6 = P1     don't care
  0x00, // R7 = P2     don't care
  0x00, // R2 = Lin    don't care
  0x04, // R3 = Lout = size to read (4 bytes)
  24,   // lines to follow

  0x21, //       %CLR_RST             ' reset = 0

// Start command mode
  0x51, //       %SET_IO              ' I/O = 1
  0x71, //       %SET_CLK             ' CLK = 1 
  0x41, //       %CLR_IO              ' I/O = 0
// Command
  0x74, //       MOV      A,0x34      ' A = 0x34 (read protection memory command)
  0x34,
  0x93, //       %WRH_R               ' write A
// Address
  0x74, //       MOV      A,0x00      ' A = 0x00 (not used)
  0x00,
  0x93, //       %WRH_R               ' write A
// Data
  0x74, //       MOV      A,0x00      ' A = 0x00 (not used)
  0x00,
  0x93, //       %WRH_R               ' write A

// Stop command mode
  0x41, //       %CLR_IO              ' I/O = 0
  0x71, //       %SET_CLK             ' CLK = 1 
  0x51, //       %SET_IO              ' I/O = 1

//Outgoing Data
  0x61, //       %CLR_CLK             ' CLK = 0
		// loop:
  0x53, //       %RDH_R               ' A   = read
  0xF6, //       MOV      @R0,A       ' *R0 = A
  0x08, //       INC      R0          ' R0++
  0xDB, //       DJNZ     R3,0xFB     ' R3--, if R3<>0 goto loop
  0xFB, //                            ' FB = FF - 4 thus move back of 4 instructions

  0xA2, //       %RST_PUL             ' reset card
  0x42  //       %RET_OK              ' data_out = [R4..R0], sw1=0x90, sw2=0x00
};

/* SpringCardS10API/LiteProS10ReadProtectionArea
 *
 * NAME
 *   LiteProS10ReadProtectionArea
 *
 * DESCRIPTION
 *   S10 read protection memory function using raw exchange
 *   For GemCore Serial Lite PRO V1.4X
 *
 * INPUTS
 *   SCARDHANDLE hCard : handle of the card
 *   BYTE recv_data[4] : 4-byte buffer to receive protection data
 *   BYTE status[2]    : 2-byte buffer to receive status code
 *
 * RETURNS
 *   SCARD_S_SUCCESS   : success, protection memory read
 *   Other code if internal or communication error has occured.  
 *
 * NOTES
 *   This function must be used with a S10 card containing a protection memory.
 *   For more information, read the datasheet of the S10 card.
 * 
 * SEE ALSO
 *   PosS10ReadProtectionArea
 *   SCardS10ReadProtectionArea
 *
 **/
LONG LiteProS10ReadProtectionArea(SCARDHANDLE hCard, BYTE recv_data[4], BYTE status[2])
{
  LONG SCardLastError;
  BYTE buffer[MAX_BYTE_TO_READ + 2];

  if (recv_data == NULL) 
    return SCARD_E_INVALID_PARAMETER;

  /* Power-up (again ?) the card */
  SCardLastError = SCardS10CardDirect(hCard, sizeof(LITE_PRO_S10_POWER_UP_CMD), LITE_PRO_S10_POWER_UP_CMD, sizeof(buffer), buffer, status);
  if (SCardLastError != SCARD_S_SUCCESS)
    return SCardLastError; 

  SCardLastError = SCardS10CardDirect(hCard, sizeof(LITE_PRO_S10_READ_PROTECTION_AREA_CMD), LITE_PRO_S10_READ_PROTECTION_AREA_CMD, sizeof(buffer), buffer, status);
  if (SCardLastError != SCARD_S_SUCCESS)
    return SCardLastError; 

  memcpy(recv_data, buffer, 4);
 
  return SCARD_S_SUCCESS; 
}

/* SpringCardS10API/LITE_PRO_S10_READ_SECURITY_AREA_CMD
 *
 * NAME
 *   LITE_PRO_S10_READ_SECURITY_AREA_CMD
 *
 * DESCRIPTION
 *   S10 read protection raw command
 *   For GemCore Serial Lite PRO V1.4X
 *
 **/
static BYTE LITE_PRO_S10_READ_SECURITY_AREA_CMD[] =
{
  0x16, // Synchronous interpreter command

  0x00, // A  = CLA    don't care
  0xB0, // R5 = INS    don't care
  0x00, // R6 = P1     don't care
  0x00, // R7 = P2     don't care
  0x00, // R2 = Lin    don't care
  0x04, // R3 = Lout = size to read (4 bytes)
  24,   // lines to follow

  0x21, //       %CLR_RST             ' reset = 0

// Start command mode
  0x51, //       %SET_IO              ' I/O = 1
  0x71, //       %SET_CLK             ' CLK = 1 
  0x41, //       %CLR_IO              ' I/O = 0
// Command
  0x74, //       MOV      A,0x31      ' A = 0x31 (read security memory command)
  0x31,
  0x93, //       %WRH_R               ' write A
// Address
  0x74, //       MOV      A,0x00      ' A = 0x00 (not used)
  0x00,
  0x93, //       %WRH_R               ' write A
// Data
  0x74, //       MOV      A,0x00      ' A = 0x00 (not used)
  0x00,
  0x93, //       %WRH_R               ' write A

// Stop command mode
  0x41, //       %CLR_IO              ' I/O = 0
  0x71, //       %SET_CLK             ' CLK = 1 
  0x51, //       %SET_IO              ' I/O = 1

//Outgoing Data
  0x61, //       %CLR_CLK             ' CLK = 0
		// loop:
  0x53, //       %RDH_R               ' A   = read
  0xF6, //       MOV      @R0,A       ' *R0 = A
  0x08, //       INC      R0          ' R0++
  0xDB, //       DJNZ     R3,0xFB     ' R3--, if R3<>0 goto loop
  0xFB, //                            ' FB = FF - 4 thus move back of 4 instructions

  0xA2, //       %RST_PUL             ' reset card
  0x42  //       %RET_OK              ' data_out = [R4..R0], sw1=0x90, sw2=0x00
};

/* SpringCardS10API/LiteProS10ReadSecurityArea
 *
 * NAME
 *   LiteProS10ReadSecurityArea
 *
 * DESCRIPTION
 *   S10 read security memory function using raw exchange
 *   For GemCore Serial Lite PRO V1.4X
 *
 * INPUTS
 *   SCARDHANDLE hCard : handle of the card
 *   BYTE recv_data[4] : 4-byte buffer to receive security data
 *   BYTE status[2]    : 2-byte buffer to receive status code
 *
 * RETURNS
 *   SCARD_S_SUCCESS   : success, security memory read
 *   Other code if internal or communication error has occured.  
 *
 * NOTES
 *   This function must be used with a S10 card containing a security memory.
 *   For more information, read the datasheet of the S10 card.
 * 
 * SEE ALSO
 *   PosS10ReadSecurityArea
 *   SCardS10ReadSecurityArea
 *
 **/
LONG LiteProS10ReadSecurityArea(SCARDHANDLE hCard, BYTE recv_data[4], BYTE status[2])
{
  LONG SCardLastError;
  BYTE buffer[MAX_BYTE_TO_READ + 2];

  if (recv_data == NULL) 
    return SCARD_E_INVALID_PARAMETER;

  /* Power-up (again ?) the card */
  SCardLastError = SCardS10CardDirect(hCard, sizeof(LITE_PRO_S10_POWER_UP_CMD), LITE_PRO_S10_POWER_UP_CMD, sizeof(buffer), buffer, status);
  if (SCardLastError != SCARD_S_SUCCESS)
    return SCardLastError; 

  SCardLastError = SCardS10CardDirect(hCard, sizeof(LITE_PRO_S10_READ_SECURITY_AREA_CMD), LITE_PRO_S10_READ_SECURITY_AREA_CMD, sizeof(buffer), buffer, status);
  if (SCardLastError != SCARD_S_SUCCESS)
    return SCardLastError; 

  memcpy(recv_data, buffer, 4);
 
  return SCARD_S_SUCCESS; 
}

/* SpringCardS10API/LITE_PRO_S10_WRITE_DATA_AREA_CMD
 *
 * NAME
 *   LITE_PRO_S10_WRITE_DATA_AREA_CMD
 *
 * DESCRIPTION
 *   S10 write data raw command
 *   For GemCore Serial Lite PRO V1.4X
 *
 **/
static BYTE LITE_PRO_S10_WRITE_DATA_AREA_CMD[] =
{
  0x16, // Synchronous interpreter command

  0x00, // A  = CLA    don't care
  0xB0, // R5 = INS    don't care
  0x00, // R6 = P1   = address
  0xFF, // R7 = P2   = data
  0x00, // R2 = Lin    don't care
  0xFF, // R3 = Lout = wait 255 clock
  23,   // lines to follow

  0x21, //       %CLR_RST             ' reset = 0

// Start command mode
  0x51, //       %SET_IO              ' I/O = 1
  0x71, //       %SET_CLK             ' CLK = 1 
  0x41, //       %CLR_IO              ' I/O = 0
// Command
  0x74, //       MOV      A,0x38      ' A = 0x38 (write data memory command)
  0x38,
  0x93, //       %WRH_R               ' write A
// Address
  0xEE, //       MOV      A,R6        ' A = R6 = P1 (write address)
  0x93, //       %WRH_R               ' write A
// Data
  0xEF, //       MOV      A,R7        ' A = R7 = P2 (data to write)
  0x93, //       %WRH_R               ' write A

// Stop command mode
  0x41, //       %CLR_IO              ' I/O = 0
  0x71, //       %SET_CLK             ' CLK = 1 
  0x51, //       %SET_IO              ' I/O = 1

//Processing
  0x61, //       %CLR_CLK             ' CLK = 0
  0x41, //       %CLR_IO              ' I/O = 0
		// loop:
  0x71, //       %SET_CLK             ' CLK = 1 
  0x61, //       %CLR_CLK             ' CLK = 0
  0xDB, //       DJNZ     R3,0xFC     ' R3--, if R3<>0 goto loop
  0xFC, //                            ' FC = FF - 3 thus move back of 3 instructions
  0x51, //       %SET_IO              ' I/O = 1

  0xA2, //       %RST_PUL             ' reset card
  0x42  //       %RET_OK              ' data_out = [R4..R0], sw1=0x90, sw2=0x00
};

/* SpringCardS10API/LiteProS10WriteDataArea
 *
 * NAME
 *   LiteProS10WriteDataArea
 *
 * DESCRIPTION
 *   S10 write data memory function using raw exchange
 *   For GemCore Serial Lite PRO V1.4X
 *
 * INPUTS
 *   SCARDHANDLE hCard : handle of the card
 *   BYTE *write_data  : data to write
 *   BYTE addr         : first address to write
 *   BYTE write_len    : length of data to write
 *   BYTE status[2]    : 2-byte buffer to receive status code
 *
 * RETURNS
 *   SCARD_S_SUCCESS   : success, data written
 *   Other code if internal or communication error has occured.  
 *
 * NOTES
 *   Maximun length of data to write is 128 bytes.
 *   If the S10 card has a PSC, this function will only be executed if a PSC has been successfully verified before.
 *   For more information, read the datasheet of the S10 card.
 * 
 * SEE ALSO
 *   PosS10WriteDataArea
 *   SCardS10WriteDataArea
 *
 **/
LONG LiteProS10WriteDataArea(SCARDHANDLE hCard, BYTE *write_data, BYTE addr, BYTE write_len, BYTE status[2])
{
  int i;
  LONG SCardLastError;
  BYTE buffer[MAX_BYTE_TO_READ + 2];

  if (write_len > MAX_BYTE_TO_READ) 
    return SCARD_E_INVALID_PARAMETER;

  if (write_data == NULL) 
    return SCARD_E_INVALID_PARAMETER;

  /* Power-up (again ?) the card */
  SCardLastError = SCardS10CardDirect(hCard, sizeof(LITE_PRO_S10_POWER_UP_CMD), LITE_PRO_S10_POWER_UP_CMD, sizeof(buffer), buffer, status);
  if (SCardLastError != SCARD_S_SUCCESS)
    return SCardLastError; 

  for (i=0 ; i < write_len ; i++)
  {
    LITE_PRO_S10_WRITE_DATA_AREA_CMD[3] = addr + i;
    LITE_PRO_S10_WRITE_DATA_AREA_CMD[4] = write_data[i];

    SCardLastError = SCardS10CardDirect(hCard, sizeof(LITE_PRO_S10_WRITE_DATA_AREA_CMD), LITE_PRO_S10_WRITE_DATA_AREA_CMD, sizeof(buffer), buffer, status);
    if (SCardLastError != SCARD_S_SUCCESS)
      return SCardLastError; 
  }
 
  return SCARD_S_SUCCESS; 
}

/* SpringCardS10API/LITE_PRO_S10_WRITE_PROTECTION_AREA_CMD
 *
 * NAME
 *   LITE_PRO_S10_WRITE_PROTECTION_AREA_CMD
 *
 * DESCRIPTION
 *   S10 write protection raw command
 *   For GemCore Serial Lite PRO V1.4X
 *
 **/
static BYTE LITE_PRO_S10_WRITE_PROTECTION_AREA_CMD[] =
{
  0x16, // Synchronous interpreter command

  0x00, // A  = CLA    don't care
  0xB0, // R5 = INS    don't care
  0x00, // R6 = P1   = address
  0xFF, // R7 = P2   = data
  0x00, // R2 = Lin    don't care
  0xFF, // R3 = Lout = wait 255 clock
  23,   // lines to follow

  0x21, //       %CLR_RST             ' reset = 0

// Start command mode
  0x51, //       %SET_IO              ' I/O = 1
  0x71, //       %SET_CLK             ' CLK = 1 
  0x41, //       %CLR_IO              ' I/O = 0
// Command
  0x74, //       MOV      A,0x3C      ' A = 0x3C (write protection memory command)
  0x3C,
  0x93, //       %WRH_R               ' write A
// Address
  0xEE, //       MOV      A,R6        ' A = R6 = P1 (write address)
  0x93, //       %WRH_R               ' write A
// Data
  0xEF, //       MOV      A,R7        ' A = R7 = P2 (data to write)
  0x93, //       %WRH_R               ' write A

// Stop command mode
  0x41, //       %CLR_IO              ' I/O = 0
  0x71, //       %SET_CLK             ' CLK = 1 
  0x51, //       %SET_IO              ' I/O = 1

//Processing
  0x61, //       %CLR_CLK             ' CLK = 0
  0x41, //       %CLR_IO              ' I/O = 0
		// loop:
  0x71, //       %SET_CLK             ' CLK = 1 
  0x61, //       %CLR_CLK             ' CLK = 0
  0xDB, //       DJNZ     R3,0xFC     ' R3--, if R3<>0 goto loop
  0xFC, //                            ' FC = FF - 3 thus move back of 3 instructions
  0x51, //       %SET_IO              ' I/O = 1

  0xA2, //       %RST_PUL             ' reset card
  0x42  //       %RET_OK              ' data_out = [R4..R0], sw1=0x90, sw2=0x00
};

/* SpringCardS10API/LiteProS10WriteProtectionArea
 *
 * NAME
 *   LiteProS10WriteProtectionArea
 *
 * DESCRIPTION
 *   S10 write protection memory function using raw exchange
 *   For GemCore Serial Lite PRO V1.4X
 *
 * WARNING
 *   The protection bytes are irreversible byte-wise.
 *
 * INPUTS
 *   SCARDHANDLE hCard : handle of the card
 *   BYTE *write_data  : data to write
 *   BYTE addr         : first address to write
 *   BYTE write_len    : length of data to write
 *   BYTE status[2]    : 2-byte buffer to receive status code
 *
 * RETURNS
 *   SCARD_S_SUCCESS   : success, data written
 *   Other code if internal or communication error has occured.  
 *
 * NOTES
 *   This function must be used with a S10 card containing a protection memory.
 *   This function will only be executed if a PSC has been successfully verified before.
 *   Maximum length of data to write is 32 bytes.
 *   Address lies between 0 and 31.
 *   For more information, read the datasheet of the S10 card.
 * 
 * SEE ALSO
 *   PosS10WriteProtectionArea
 *   SCardS10WriteProtectionArea
 *
 **/
LONG LiteProS10WriteProtectionArea(SCARDHANDLE hCard, BYTE *write_data, BYTE addr, BYTE write_len, BYTE status[2])
{
  int i;
  LONG SCardLastError;
  BYTE buffer[MAX_BYTE_TO_READ + 2];

  if (write_len > 32) 
    return SCARD_E_INVALID_PARAMETER;

  if (write_data == NULL) 
    return SCARD_E_INVALID_PARAMETER;

  if (! ((addr >= 0x00) && (addr <= 0x1F)))
    return SCARD_E_INVALID_PARAMETER;

  /* Power-up (again ?) the card */
  SCardLastError = SCardS10CardDirect(hCard, sizeof(LITE_PRO_S10_POWER_UP_CMD), LITE_PRO_S10_POWER_UP_CMD, sizeof(buffer), buffer, status);
  if (SCardLastError != SCARD_S_SUCCESS)
    return SCardLastError; 

  for (i=0 ; i < write_len ; i++)
  {
    LITE_PRO_S10_WRITE_PROTECTION_AREA_CMD[3] = addr + i;
    LITE_PRO_S10_WRITE_PROTECTION_AREA_CMD[4] = write_data[i];

    SCardLastError = SCardS10CardDirect(hCard, sizeof(LITE_PRO_S10_WRITE_PROTECTION_AREA_CMD), LITE_PRO_S10_WRITE_PROTECTION_AREA_CMD, sizeof(buffer), buffer, status);
    if (SCardLastError != SCARD_S_SUCCESS)
      return SCardLastError; 
  }
 
  return SCARD_S_SUCCESS; 
}

/* SpringCardS10API/LITE_PRO_S10_UPDATE_SECURITY_AREA_CMD
 *
 * NAME
 *   LITE_PRO_S10_UPDATE_SECURITY_AREA_CMD
 *
 * DESCRIPTION
 *   S10 update security raw command
 *   For GemCore Serial Lite PRO V1.4X
 *
 **/
static BYTE LITE_PRO_S10_UPDATE_SECURITY_AREA_CMD[] =
{
  0x16, // Synchronous interpreter command

  0x00, // A  = CLA    don't care
  0xB0, // R5 = INS    don't care
  0x00, // R6 = P1   = address
  0xFF, // R7 = P2   = data
  0x00, // R2 = Lin    don't care
  0xFF, // R3 = Lout = wait 255 clock
  23,   // lines to follow

  0x21, //       %CLR_RST             ' reset = 0

// Start command mode
  0x51, //       %SET_IO              ' I/O = 1
  0x71, //       %SET_CLK             ' CLK = 1 
  0x41, //       %CLR_IO              ' I/O = 0
// Command
  0x74, //       MOV      A,0x39      ' A = 0x39 (update security memory command)
  0x39,
  0x93, //       %WRH_R               ' write A
// Address
  0xEE, //       MOV      A,R6        ' A = R6 = P1 (write address)
  0x93, //       %WRH_R               ' write A
// Data
  0xEF, //       MOV      A,R7        ' A = R7 = P2 (data to update)
  0x93, //       %WRH_R               ' write A

// Stop command mode
  0x41, //       %CLR_IO              ' I/O = 0
  0x71, //       %SET_CLK             ' CLK = 1 
  0x51, //       %SET_IO              ' I/O = 1

//Processing
  0x61, //       %CLR_CLK             ' CLK = 0
  0x41, //       %CLR_IO              ' I/O = 0
		// loop:
  0x71, //       %SET_CLK             ' CLK = 1 
  0x61, //       %CLR_CLK             ' CLK = 0
  0xDB, //       DJNZ     R3,0xFC     ' R3--, if R3<>0 goto loop
  0xFC, //                            ' FC = FF - 3 thus move back of 3 instructions
  0x51, //       %SET_IO              ' I/O = 1

  0xA2, //       %RST_PUL             ' reset card
  0x42  //       %RET_OK              ' data_out = [R4..R0], sw1=0x90, sw2=0x00
};

/* SpringCardS10API/LiteProS10UpdateSecurityArea
 *
 * NAME
 *   LiteProS10UpdateSecurityArea
 *
 * DESCRIPTION
 *   S10 update security memory function using raw exchange
 *   For GemCore Serial Lite PRO V1.4X
 *
 * INPUTS
 *   SCARDHANDLE hCard : handle of the card
 *   BYTE *update_data : data to update
 *   BYTE addr         : first address to update
 *   BYTE update_len   : length of data to update
 *   BYTE status[2]    : 2-byte buffer to receive status code
 *
 * RETURNS
 *   SCARD_S_SUCCESS   : success, data updated
 *   Other code if internal or communication error has occured.  
 *
 * NOTES
 *   This function must be used with a S10 card containing a security memory.
 *   This function will only be executed if a PSC has been successfully verified before.
 *   Maximum length of data to write is 4 bytes.
 *   Address lies between 0 and 3.
 *   For more information, read the datasheet of the S10 card.
 * 
 * SEE ALSO
 *   PosS10UpdateSecurityArea
 *   SCardS10UpdateSecurityArea
 *
 **/
LONG LiteProS10UpdateSecurityArea(SCARDHANDLE hCard, BYTE *update_data, BYTE addr, BYTE update_len, BYTE status[2])
{
  int i;
  LONG SCardLastError;
  BYTE buffer[MAX_BYTE_TO_READ + 2];

  if (update_len > 4) 
    return SCARD_E_INVALID_PARAMETER;

  if (update_data == NULL) 
    return SCARD_E_INVALID_PARAMETER;

  if (! ((addr >= 0x00) && (addr <= 0x03)))
    return SCARD_E_INVALID_PARAMETER;

  /* Power-up (again ?) the card */
  SCardLastError = SCardS10CardDirect(hCard, sizeof(LITE_PRO_S10_POWER_UP_CMD), LITE_PRO_S10_POWER_UP_CMD, sizeof(buffer), buffer, status);
  if (SCardLastError != SCARD_S_SUCCESS)
    return SCardLastError; 

  for (i=0 ; i < update_len ; i++)
  {
    LITE_PRO_S10_UPDATE_SECURITY_AREA_CMD[3] = addr + i;
    LITE_PRO_S10_UPDATE_SECURITY_AREA_CMD[4] = update_data[i];

    SCardLastError = SCardS10CardDirect(hCard, sizeof(LITE_PRO_S10_UPDATE_SECURITY_AREA_CMD), LITE_PRO_S10_UPDATE_SECURITY_AREA_CMD, sizeof(buffer), buffer, status);
    if (SCardLastError != SCARD_S_SUCCESS)
      return SCardLastError; 
  }
 
  return SCARD_S_SUCCESS; 
}

/* SpringCardS10API/LITE_PRO_S10_COMPARE_VERIFICATION_CMD
 *
 * NAME
 *   LITE_PRO_S10_COMPARE_VERIFICATION_CMD
 *
 * DESCRIPTION
 *   S10 compare verification data raw command
 *   For GemCore Serial Lite PRO V1.4X
 *
 **/
static BYTE LITE_PRO_S10_COMPARE_VERIFICATION_CMD[] =
{
  0x16, // Synchronous interpreter command

  0x00, // A  = CLA    don't care
  0xB0, // R5 = INS    don't care
  0x00, // R6 = P1   = address
  0xFF, // R7 = P2   = code
  0x00, // R2 = Lin    don't care
  0x02, // R3 = Lout = wait 2 clock
  23,   // lines to follow

  0x21, //       %CLR_RST             ' reset = 0

// Start command mode
  0x51, //       %SET_IO              ' I/O = 1
  0x71, //       %SET_CLK             ' CLK = 1 
  0x41, //       %CLR_IO              ' I/O = 0
// Command
  0x74, //       MOV      A,0x33      ' A = 0x33 (compare verification data command)
  0x33,
  0x93, //       %WRH_R               ' write A
// Address
  0xEE, //       MOV      A,R6        ' A = R6 = P1 (write address)
  0x93, //       %WRH_R               ' write A
// Code
  0xEF, //       MOV      A,R7        ' A = R7 = P2 (code to compare)
  0x93, //       %WRH_R               ' write A

// Stop command mode
  0x41, //       %CLR_IO              ' I/O = 0
  0x71, //       %SET_CLK             ' CLK = 1 
  0x51, //       %SET_IO              ' I/O = 1

//Processing
  0x61, //       %CLR_CLK             ' CLK = 0
  0x41, //       %CLR_IO              ' I/O = 0
		// loop:
  0x71, //       %SET_CLK             ' CLK = 1 
  0x61, //       %CLR_CLK             ' CLK = 0
  0xDB, //       DJNZ     R3,0xFC     ' R3--, if R3<>0 goto loop
  0xFC, //                            ' FC = FF - 3 thus move back of 3 instructions
  0x51, //       %SET_IO              ' I/O = 1

  0xA2, //       %RST_PUL             ' reset card
  0x42  //       %RET_OK              ' data_out = [R4..R0], sw1=0x90, sw2=0x00
};

/* SpringCardS10API/LiteProS10PSCVerification
 *
 * NAME
 *   LiteProS10PSCVerification
 *
 * DESCRIPTION
 *   S10 verification function of the Programmable Security Code (PSC)  using raw exchange
 *   For GemCore Serial Lite PRO V1.4X
 *
 * INPUTS
 *   SCARDHANDLE hCard   : handle of the card
 *   BYTE code3          : byte 3 of Programmable Security Code
 *   BYTE code2          : byte 2 of Programmable Security Code
 *   BYTE code1          : byte 1 of Programmable Security Code
 *   BYTE status[2]      : 2-byte buffer to receive status code
 *
 * RETURNS
 *   SCARD_S_SUCCESS     : PSC verification succeeded
 *   SCARD_W_CHV_BLOCKED : The card cannot be accessed because the maximum number of PIN entry 
 *                         attempts has been reached.
 *   Other code if internal or communication error has occured.  
 *
 * NOTES
 *   This function must be used with a S10 card containing a Programmable Security Code.
 *   For more information, read the datasheet of the S10 card.
 * 
 * SEE ALSO
 *   PosS10PSCVerification
 *   SCardS10PSCVerification
 *
 **/
LONG LiteProS10PSCVerification(SCARDHANDLE hCard, BYTE code3, BYTE code2, BYTE code1, BYTE status[2])
{
  LONG SCardLastError;
  BYTE buffer[MAX_BYTE_TO_READ + 2];
  BYTE inputData;

  /* Power-up (again ?) the card */
  SCardLastError = SCardS10CardDirect(hCard, sizeof(LITE_PRO_S10_POWER_UP_CMD), LITE_PRO_S10_POWER_UP_CMD, sizeof(buffer), buffer, status);
  if (SCardLastError != SCARD_S_SUCCESS)
    return SCardLastError; 

  SCardLastError = SCardS10CardDirect(hCard, sizeof(LITE_PRO_S10_READ_SECURITY_AREA_CMD), LITE_PRO_S10_READ_SECURITY_AREA_CMD, sizeof(buffer), buffer, status);
  if (SCardLastError != SCARD_S_SUCCESS)
    return SCardLastError; 

  if ((buffer[0] & 0x01) == 0x01)
    inputData = 0xFE;
  else if ((buffer[0] & 0x02) == 0x02)
    inputData = 0xFD;
  else if ((buffer[0] & 0x04) == 0x04)
    inputData = 0xFB;
  else
	return SCARD_W_CHV_BLOCKED;

  LITE_PRO_S10_UPDATE_SECURITY_AREA_CMD[3] = 0x00;
  LITE_PRO_S10_UPDATE_SECURITY_AREA_CMD[4] = inputData;
  
  SCardLastError = SCardS10CardDirect(hCard, sizeof(LITE_PRO_S10_UPDATE_SECURITY_AREA_CMD), LITE_PRO_S10_UPDATE_SECURITY_AREA_CMD, sizeof(buffer), buffer, status);
  if (SCardLastError != SCARD_S_SUCCESS)
    return SCardLastError; 

  LITE_PRO_S10_COMPARE_VERIFICATION_CMD[3] = 0x01;
  LITE_PRO_S10_COMPARE_VERIFICATION_CMD[4] = code1;

  SCardLastError = SCardS10CardDirect(hCard, sizeof(LITE_PRO_S10_COMPARE_VERIFICATION_CMD), LITE_PRO_S10_COMPARE_VERIFICATION_CMD, sizeof(buffer), buffer, status);
  if (SCardLastError != SCARD_S_SUCCESS)
    return SCardLastError; 

  LITE_PRO_S10_COMPARE_VERIFICATION_CMD[3] = 0x02;
  LITE_PRO_S10_COMPARE_VERIFICATION_CMD[4] = code2;

  SCardLastError = SCardS10CardDirect(hCard, sizeof(LITE_PRO_S10_COMPARE_VERIFICATION_CMD), LITE_PRO_S10_COMPARE_VERIFICATION_CMD, sizeof(buffer), buffer, status);
  if (SCardLastError != SCARD_S_SUCCESS)
    return SCardLastError; 

  LITE_PRO_S10_COMPARE_VERIFICATION_CMD[3] = 0x03;
  LITE_PRO_S10_COMPARE_VERIFICATION_CMD[4] = code3;

  SCardLastError = SCardS10CardDirect(hCard, sizeof(LITE_PRO_S10_COMPARE_VERIFICATION_CMD), LITE_PRO_S10_COMPARE_VERIFICATION_CMD, sizeof(buffer), buffer, status);
  if (SCardLastError != SCARD_S_SUCCESS)
    return SCardLastError; 

  LITE_PRO_S10_UPDATE_SECURITY_AREA_CMD[3] = 0x00;
  LITE_PRO_S10_UPDATE_SECURITY_AREA_CMD[4] = 0xFF;

  SCardLastError = SCardS10CardDirect(hCard, sizeof(LITE_PRO_S10_UPDATE_SECURITY_AREA_CMD), LITE_PRO_S10_UPDATE_SECURITY_AREA_CMD, sizeof(buffer), buffer, status);
  if (SCardLastError != SCARD_S_SUCCESS)
    return SCardLastError; 
 
  return SCARD_S_SUCCESS; 
}
