/* SpringCardS9API/S9_GCR_1.5X
 *
 * NAME
 *   SpringCardS9API :: S9 Card specific functions for GemCore POS Serial V1.5X
 *
 * COPYRIGHT
 *   (c) 2009 SpringCard - www.springcard.com
 *
 * DESCRIPTION
 *   Implementation of S9 functions to read or write the card
 *   For GemCore POS Serial V1.5X
 *
 **/

#include "springcard_s9_i.h"

/* SpringCardS9API/POS_S9_READ_DATA_AREA_CMD
 *
 * NAME
 *   POS_S9_READ_DATA_AREA_CMD
 *
 * DESCRIPTION
 *   S9 read data APDU command
 *   For GemCore POS Serial V1.5X
 *
 **/
static BYTE POS_S9_READ_DATA_AREA_CMD[] =
{
  0x00, 0xB0, 0x00 /* Address H */, 0x00 /* Address L */, 0x00 /* Read length */ 	
};

/* SpringCardS9API/PosS9ReadDataArea
 *
 * NAME
 *   PosS9ReadDataArea
 *
 * DESCRIPTION
 *   S9 read main memory function using APDU exchange
 *   For GemCore POS Serial V1.5X
 *
 * INPUTS
 *   SCARDHANDLE hCard : handle of the card
 *   BYTE *recv_data   : data received
 *   WORD addr         : first address to read
 *   BYTE recv_len     : length of data to receive
 *   BYTE status[2]    : 2-byte buffer to receive status code
 *
 * RETURNS
 *   SCARD_S_SUCCESS   : success, card read
 *   Other code if internal or communication error has occured.  
 *
 * NOTES
 *   Maximun length of data to read is 128 bytes
 *
 * SEE ALSO
 *   LiteProS9ReadDataArea
 *   ScardS9ReadDataArea
 *
 **/
LONG PosS9ReadDataArea(SCARDHANDLE hCard, BYTE *recv_data, WORD addr, BYTE recv_len, BYTE status[2])
{ 
  LONG SCardLastError;
  BYTE buffer[MAX_BYTE_TO_READ + 2];

  if (recv_len > MAX_BYTE_TO_READ) 
    return SCARD_E_INVALID_PARAMETER;

  if (recv_data == NULL) 
    return SCARD_E_INVALID_PARAMETER;
    
  POS_S9_READ_DATA_AREA_CMD[2] = (BYTE) (addr / 0x0100);
	POS_S9_READ_DATA_AREA_CMD[3] = (BYTE) (addr % 0x0100);
	POS_S9_READ_DATA_AREA_CMD[4] = recv_len;

  SCardLastError = SCardS9CardTransmit(hCard, sizeof(POS_S9_READ_DATA_AREA_CMD), POS_S9_READ_DATA_AREA_CMD, sizeof(buffer), buffer, status);
  if (SCardLastError != SCARD_S_SUCCESS)
    return SCardLastError; 

  memcpy(recv_data, buffer, recv_len);

  return SCARD_S_SUCCESS;
}

/* SpringCardS9API/POS_S9_READ_PROTECTION_AREA_CMD
 *
 * NAME
 *   POS_S9_READ_PROTECTION_AREA_CMD
 *
 * DESCRIPTION
 *   S9 read protection APDU command
 *   For GemCore POS Serial V1.5X
 *
 **/
static BYTE POS_S9_READ_PROTECTION_AREA_CMD[] =
{
  0x00, 0xB0, 0x80 /* 0x80 + Address H */, 0x00, 0x20	
};

/* SpringCardS9API/PosS9ReadProtectionArea
 *
 * NAME
 *   PosS9ReadProtectionArea
 *
 * DESCRIPTION
 *   S9 read protection memory function using APDU exchange
 *   For GemCore POS Serial V1.5X
 *
 * INPUTS
 *   SCARDHANDLE hCard  : handle of the card
 *   BYTE recv_data[32] : 32-byte buffer to receive protection data
 *   WORD addr          : first address to read
 *   BYTE status[2]     : 2-byte buffer to receive status code
 *
 * RETURNS
 *   SCARD_S_SUCCESS   : success, protection memory read
 *   Other code if internal or communication error has occured.  
 *
 * NOTES
 *   This function must be used with a S9 card containing a protection memory.
 *   For more information, read the datasheet of the S9 card.
 * 
 * SEE ALSO
 *   LiteProS9ReadProtectionArea
 *   SCardS9ReadProtectionArea
 *
 **/
LONG PosS9ReadProtectionArea(SCARDHANDLE hCard, BYTE recv_data[32], WORD addr, BYTE status[2])
{ 
  LONG SCardLastError;
  BYTE buffer[MAX_BYTE_TO_READ + 2];

  if (recv_data == NULL) 
    return SCARD_E_INVALID_PARAMETER;
    
  POS_S9_READ_PROTECTION_AREA_CMD[2] += (BYTE) (addr / 0x0100);
	POS_S9_READ_PROTECTION_AREA_CMD[3]  = (BYTE) (addr % 0x0100);

  SCardLastError = SCardS9CardTransmit(hCard, sizeof(POS_S9_READ_PROTECTION_AREA_CMD), POS_S9_READ_PROTECTION_AREA_CMD, sizeof(buffer), buffer, status);
  if (SCardLastError != SCARD_S_SUCCESS)
    return SCardLastError; 

  memcpy(recv_data, buffer, 32);

  return SCARD_S_SUCCESS;
}

/* SpringCardS9API/POS_S9_READ_SECURITY_AREA_CMD
 *
 * NAME
 *   POS_S9_READ_SECURITY_AREA_CMD
 *
 * DESCRIPTION
 *   S9 read security APDU command
 *   For GemCore POS Serial V1.5X
 *
 **/
static BYTE POS_S9_READ_SECURITY_AREA_CMD[] =
{
  0x00, 0xB0, 0xC0, 0x00, 0x03	
};

/* SpringCardS9API/PosS9ReadSecurityArea
 *
 * NAME
 *   PosS9ReadSecurityArea
 *
 * DESCRIPTION
 *   S9 read security memory function using APDU exchange
 *   For GemCore POS Serial V1.5X
 *
 * INPUTS
 *   SCARDHANDLE hCard : handle of the card
 *   BYTE recv_data[3] : 3-byte buffer to receive security data
 *   BYTE status[2]    : 2-byte buffer to receive status code
 *
 * RETURNS
 *   SCARD_S_SUCCESS   : success, security memory read
 *   Other code if internal or communication error has occured.  
 *
 * NOTES
 *   This function must be used with a S9 card containing a security memory.
 *   For more information, read the datasheet of the S9 card.
 * 
 * SEE ALSO
 *   LiteProS9ReadSecurityArea
 *   SCardS9ReadSecurityArea
 *
 **/
LONG PosS9ReadSecurityArea(SCARDHANDLE hCard, BYTE recv_data[3], BYTE status[2])
{ 
  LONG SCardLastError;
  BYTE buffer[MAX_BYTE_TO_READ + 2];

  if (recv_data == NULL) 
    return SCARD_E_INVALID_PARAMETER;

  SCardLastError = SCardS9CardTransmit(hCard, sizeof(POS_S9_READ_SECURITY_AREA_CMD), POS_S9_READ_SECURITY_AREA_CMD, sizeof(buffer), buffer, status);
  if (SCardLastError != SCARD_S_SUCCESS)
    return SCardLastError; 

  memcpy(recv_data, buffer, 3);

  return SCARD_S_SUCCESS;
}

/* SpringCardS9API/POS_S9_WRITE_DATA_AREA_CMD
 *
 * NAME
 *   POS_S9_WRITE_DATA_AREA_CMD
 *
 * DESCRIPTION
 *   S9 write data APDU command
 *   For GemCore POS Serial V1.5X
 *
 **/
static BYTE POS_S9_WRITE_DATA_AREA_CMD[] =
{
  0x00, 0xD0, 0x00 /* Address H */, 0x00 /* Address L */, 0x00 /* Write length*/ 
};

/* SpringCardS9API/PosS9WriteDataArea
 *
 * NAME
 *   PosS9WriteDataArea
 *
 * DESCRIPTION
 *   S9 write data memory function using APDU exchange
 *   For GemCore POS Serial V1.5X
 *
 * INPUTS
 *   SCARDHANDLE hCard : handle of the card
 *   BYTE *write_data  : data to write
 *   WORD addr         : first address to write
 *   BYTE write_len    : length of data to write
 *   BYTE status[2]    : 2-byte buffer to receive status code
 *
 * RETURNS
 *   SCARD_S_SUCCESS   : success, data written
 *   Other code if internal or communication error has occured.  
 *
 * NOTES
 *   Maximun length of data to write is 128 bytes.
 *   If the S9 card has a PSC, this function will only be executed if a PSC has been successfully verified before.
 *   For more information, read the datasheet of the S9 card.
 * 
 * SEE ALSO
 *   LiteProS9WriteDataArea
 *   SCardS9WriteDataArea
 *
 **/
LONG PosS9WriteDataArea(SCARDHANDLE hCard, BYTE *write_data, WORD addr, BYTE write_len, BYTE status[2])
{ 
  int i;
  BYTE buffer_len = 0;
  BYTE buffer[MAX_BYTE_TO_READ + 7];

  if (write_len > MAX_BYTE_TO_READ) 
    return SCARD_E_INVALID_PARAMETER;

  if (write_data == NULL) 
    return SCARD_E_INVALID_PARAMETER;

  POS_S9_WRITE_DATA_AREA_CMD[2] = (BYTE) (addr / 0x0100);
  POS_S9_WRITE_DATA_AREA_CMD[3] = (BYTE) (addr % 0x0100);
  POS_S9_WRITE_DATA_AREA_CMD[4] = write_len;

  for (i=0 ; i < sizeof(POS_S9_WRITE_DATA_AREA_CMD) ; i++)
	  buffer[i] = POS_S9_WRITE_DATA_AREA_CMD[i];
  for (i=0 ; i < write_len ; i++)
	  buffer[sizeof(POS_S9_WRITE_DATA_AREA_CMD) + i] = write_data[i];

  buffer_len = sizeof(POS_S9_WRITE_DATA_AREA_CMD) + write_len;
  if (!SCardS9CardTransmit(hCard, buffer_len, buffer, sizeof(buffer), buffer, status))
      return FALSE;

  return TRUE;
}

/* SpringCardS9API/POS_S9_WRITE_PROTECTION_AREA_CMD
 *
 * NAME
 *   POS_S9_WRITE_PROTECTION_AREA_CMD
 *
 * DESCRIPTION
 *   S9 write protection APDU command
 *   For GemCore POS Serial V1.5X
 *
 **/
static BYTE POS_S9_WRITE_PROTECTION_AREA_CMD[] =
{
  0x00, 0xD0, 0x80 /* 0x80 + Address H */, 0x00 /* Address L */, 0x01		
};

/* SpringCardS9API/PosS9WriteProtectionArea
 *
 * NAME
 *   PosS9WriteProtectionArea
 *
 * DESCRIPTION
 *   S9 write protection memory function using APDU exchange
 *   For GemCore POS Serial V1.5X
 *
 * WARNING
 *   The protection bytes are irreversible byte-wise.
 *
 * INPUTS
 *   SCARDHANDLE hCard : handle of the card
 *   BYTE *write_data  : data to write
 *   WORD addr         : first address to write
 *   BYTE write_len    : length of data to write
 *   BYTE status[2]    : 2-byte buffer to receive status code
 *
 * RETURNS
 *   SCARD_S_SUCCESS   : success, data written
 *   Other code if internal or communication error has occured.  
 *
 * NOTES
 *   This function must be used with a S9 card containing a protection memory.
 *   This function will only be executed if a PSC has been successfully verified before.
 *   For more information, read the datasheet of the S9 card.
 * 
 * SEE ALSO
 *   LiteProS9WriteProtectionArea
 *   SCardS9WriteProtectionArea
 *
 **/
LONG PosS9WriteProtectionArea(SCARDHANDLE hCard, BYTE *write_data, WORD addr, BYTE write_len, BYTE status[2])
{ 
  int i;
  int index;
  BYTE buffer_len = 0;
  BYTE buffer[MAX_BYTE_TO_READ + 7];
  WORD cur_addr;
	WORD len_left;

  if (write_data == NULL) 
    return SCARD_E_INVALID_PARAMETER;

  index = 0;
	cur_addr = addr;
	len_left = write_len;

	while (len_left) 
	{
		POS_S9_WRITE_PROTECTION_AREA_CMD[2] += (BYTE) (cur_addr / 0x0100);
	  POS_S9_WRITE_PROTECTION_AREA_CMD[3]  = (BYTE) (cur_addr % 0x0100);

		for (i=0 ; i < sizeof(POS_S9_WRITE_PROTECTION_AREA_CMD) ; i++)
		  buffer[i] = POS_S9_WRITE_PROTECTION_AREA_CMD[i];
		
		buffer[sizeof(POS_S9_WRITE_PROTECTION_AREA_CMD)] = write_data[index];

    buffer_len = sizeof(POS_S9_WRITE_PROTECTION_AREA_CMD) + 1;
    if (!SCardS9CardTransmit(hCard, buffer_len, buffer, sizeof(buffer), buffer, status))
      return FALSE;
      
    index++;
		cur_addr++;
		len_left--;
	}

  return TRUE;
}

/* SpringCardS9API/POS_S9_UPDATE_SECURITY_AREA_CMD
 *
 * NAME
 *   POS_S9_UPDATE_SECURITY_AREA_CMD
 *
 * DESCRIPTION
 *   S9 update security APDU command
 *   For GemCore POS Serial V1.5X
 *
 **/
static BYTE POS_S9_UPDATE_SECURITY_AREA_CMD[] =
{
  0x00, 0xD0, 0xC0, 0x00 /* Address */, 0x00 /* Write length*/ 	
};

/* SpringCardS9API/PosS9UpdateSecurityArea
 *
 * NAME
 *   PosS9UpdateSecurityArea
 *
 * DESCRIPTION
 *   S9 update security memory function using APDU exchange
 *   For GemCore POS Serial V1.5X
 *
 * INPUTS
 *   SCARDHANDLE hCard : handle of the card
 *   BYTE *update_data : data to update
 *   WORD addr         : first address to update
 *   BYTE update_len   : length of data to update
 *   BYTE status[2]    : 2-byte buffer to receive status code
 *
 * RETURNS
 *   SCARD_S_SUCCESS   : success, data updated
 *   Other code if internal or communication error has occured.  
 *
 * NOTES
 *   This function must be used with a S9 card containing a security memory.
 *   This function will only be executed if a PSC has been successfully verified before.
 *   For more information, read the datasheet of the S9 card.
 * 
 * SEE ALSO
 *   LiteProS9UpdateSecurityArea
 *   SCardS9UpdateSecurityArea
 *
 **/
LONG PosS9UpdateSecurityArea(SCARDHANDLE hCard, BYTE *update_data, WORD addr, BYTE update_len, BYTE status[2])
{ 
  int i;
  BYTE buffer_len = 0;
  BYTE buffer[MAX_BYTE_TO_READ + 7];

  if (update_data == NULL) 
    return SCARD_E_INVALID_PARAMETER;
 
  POS_S9_UPDATE_SECURITY_AREA_CMD[2] = (BYTE) (addr / 0x0100);
	POS_S9_UPDATE_SECURITY_AREA_CMD[3] = (BYTE) (addr % 0x0100);
  POS_S9_UPDATE_SECURITY_AREA_CMD[4] = update_len;

  for (i=0 ; i < sizeof(POS_S9_UPDATE_SECURITY_AREA_CMD) ; i++)
	  buffer[i] = POS_S9_UPDATE_SECURITY_AREA_CMD[i];
  for (i=0 ; i < update_len ; i++)
	  buffer[sizeof(POS_S9_UPDATE_SECURITY_AREA_CMD) + i] = update_data[i];

  buffer_len = sizeof(POS_S9_UPDATE_SECURITY_AREA_CMD) + update_len;
  if (!SCardS9CardTransmit(hCard, buffer_len, buffer, sizeof(buffer), buffer, status))
      return FALSE;

  return TRUE;
}

/* SpringCardS9API/POS_S9_PSC_VERIFICATION_CMD
 *
 * NAME
 *   POS_S9_PSC_VERIFICATION_CMD
 *
 * DESCRIPTION
 *   S9 PSC verification APDU command
 *   For GemCore POS Serial V1.5X
 *
 **/
static BYTE POS_S9_PSC_VERIFICATION_CMD[] =
{
  0x00, 0x20, 0x00, 0x00, 0x02 , 0x00 /* Code 2*/ , 0x00 /* Code 1*/	
};

/* SpringCardS9API/PosS9PSCVerification
 *
 * NAME
 *   PosS9PSCVerification
 *
 * DESCRIPTION
 *   S9 verification function of the Programmable Security Code (PSC) using APDU exchange
 *   For GemCore POS Serial V1.5X
 *
 * INPUTS
 *   SCARDHANDLE hCard : handle of the card
 *   BYTE code2        : byte 2 of Programmable Security Code
 *   BYTE code1        : byte 1 of Programmable Security Code
 *   BYTE status[2]    : 2-byte buffer to receive status code
 *
 * RETURNS
 *   SCARD_S_SUCCESS   : PSC verification succeeded
 *   Other code if internal or communication error has occured.  
 *
 * NOTES
 *   This function must be used with a S9 card containing a Programmable Security Code.
 *   For more information, read the datasheet of the S9 card.
 * 
 * SEE ALSO
 *   LiteProS9PSCVerification
 *   SCardS9PSCVerification
 *
 **/
LONG PosS9PSCVerification(SCARDHANDLE hCard, BYTE code2, BYTE code1, BYTE status[2])
{ 
  LONG SCardLastError;
  BYTE buffer[MAX_BYTE_TO_READ + 2];

  POS_S9_PSC_VERIFICATION_CMD[5] = code2;
  POS_S9_PSC_VERIFICATION_CMD[6] = code1;

  SCardLastError = SCardS9CardTransmit(hCard, sizeof(POS_S9_PSC_VERIFICATION_CMD), POS_S9_PSC_VERIFICATION_CMD, sizeof(buffer), buffer, status);
  if (SCardLastError != SCARD_S_SUCCESS)
    return SCardLastError; 

  return SCARD_S_SUCCESS;
}